#!/usr/bin/env ruby
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version. This program is distributed in the hope that
# it will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
# the GNU General Public License for more details. You should have
# received a copy of the GNU General Public License along with this
# program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place - Suite 330, Boston, MA 02111-1307, USA.
# 
# Log in to gmail and get a count of the number of new unread 
# emails waiting
#
# Based on: http://tvg.ca/files/gmail.phps
# 

require 'net/https'
require 'cgi'

class GMailConnection

 private

  def proxy_class
        gconf_proxy_root = '/system/http_proxy'

        if !$gconf["#{gconf_proxy_root}/use_http_proxy"] then
                Net::HTTP
        else
                username, password = nil, nil

                if $gconf["#{gconf_proxy_root}/use_authentication"] then
                        username = $gconf["#{gconf_proxy_root}/authentication_user"]
                        password = $gconf["#{gconf_proxy_root}/authentication_password"]
                end

                host = $gconf["#{gconf_proxy_root}/host"]
                port = $gconf["#{gconf_proxy_root}/port"]

                Net::HTTP.Proxy(host, port, username, password)
        end
 end

 def postdata_to_string(postdata)
	postdata_string = ''

	postdata.each_pair do |key, val|
		if postdata_string.length > 0
			postdata_string += '&'
		end
		postdata_string += "#{key}=#{CGI::escape(val)}"
	end

	postdata_string
 end

 def login
	postdata = {
		"Email" => @username,
		"Passwd" => @password,
		"null" => "Sign in",
		"continue" => "https://gmail.google.com/gmail",
		"service" => "mail",
	}

	@conn = proxy_class.new('www.google.com', 443)
	@conn.use_ssl = true

	resp, data = @conn.post('/accounts/ServiceLoginBoxAuth', 
			       postdata_to_string(postdata), nil)

	resp['set-cookie'] =~ /(SID\=.*?\=)/

	@cookie = $1

	data =~ /CheckCookie.*_sgh%3D([^&]+)&amp;/
	sgh=$1
       
	@conn = proxy_class.new('gmail.google.com', 443)
	@conn.use_ssl = true
               
	resp, data = @conn.get("/gmail?_sgh=#{sgh}&service=mail&html=LoginDoneHtml", {"Cookie" => @cookie})
       
	resp['set-cookie'] =~ /(SID\=.*?;)/
	@cookie = $1 
	resp['set-cookie'] =~ /(GV\=.*?;)/
	@cookie += $1 
	resp['set-cookie'] =~ /(GMAIL_AT\=.*?;)/
	@cookie += $1 
	resp['set-cookie'] =~ /(S\=.*?;)/
	@cookie += $1 
       
	if @cookie =~ /GV=;/ then
		raise "Login failed"
	end
 end

 public
 
 def initialize(username, password)
	@username = username
	@password = password
	login
 end

 def get_page(page)
	headers = {
		"Cookie" => @cookie
	}

	resp, data = @conn.get(page, headers)

	data
 end

 def check_mail(non_inbox = true)

	query = "is%3Aunread"

	if !non_inbox then
		query += "%20in%3Ainbox"
	end

	req = "/gmail?search=query&q=#{query}&view=tl&start=0"

	s = get_page(req)

	if s !~ /Search results for: .*?,(\d+),/ then
		raise "Error checking mail"
	end

	$1.to_i
 end

end

